## P1B1: Autoencoder Compressed Representation for Gene Expression

**Overview**: Given a sample of gene expression data, build a sparse autoencoder that can compress the expression profile into a low-dimensional vector.

**Relationship to core problem**: Many molecular assays generate large numbers of features that can lead to time-consuming processing and over-fitting in learning tasks; hence, a core capability we intend to build is feature reduction.

**Expected outcome**: Build an autoencoder that collapse high dimensional expression profiles into low dimensional vectors without much loss of information.

### Benchmark Specs Requirements 

#### Description of the Data
* Data source: RNA-seq data from GDC 
* Input dimensions: 60,484 floats; log(1+x) transformed FPKM-UQ values
* Output dimensions: Same as input
* Latent representation dimension: 1000
* Sample size: 4,000 (3000 training + 1000 test)
* Notes on data balance and other issues: unlabeled data draw from a diverse set of cancer types

#### Expected Outcomes
* Reconstructed expression profiles
* Output range: float; same as log transformed input

#### Evaluation Metrics
* Accuracy or loss function: mean squared error
* Expected performance of a naïve method: landmark genes picked by linear regression 

#### Description of the Network
* Proposed network architecture: MLP with encoding layers, dropout layers, bottleneck layer, and decoding layers
* Number of layers: At least three hidden layers including one encoding layer, one bottleneck layer, and one decoding layer

### Running the baseline implementation

```
cd Pilot1/P1B1
python p1b1_baseline_keras2.py
```
The training and test data files will be downloaded the first time this is run and will be cached for future runs. 